<?php

declare(strict_types=1);

namespace Drupal\cutoff_date\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a cut-off date banner block.
 *
 * @Block(
 *   id = "cutoff_date_banner",
 *   admin_label = @Translation("Cut-off Date Banner"),
 *   category = @Translation("System"),
 * )
 */
final class CutOffDateBannerBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs the plugin instance.
   */
  public function __construct(
    array $configuration,
          $plugin_id,
          $plugin_definition,
    private readonly ConfigFactoryInterface $configFactory,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'banner_text' => [
        'value' => '',
        'format' => '',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    $form['banner_text'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Override Cut-off Date banner text'),
      '#default_value' => $this->configuration['banner_text']['value'],
      '#format' => $this->configuration['banner_text']['format'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state): void {
    $this->configuration['banner_text'] = $form_state->getValue('banner_text');
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $banner_text = $this->configuration['banner_text'];
    if (empty($banner_text['value'])) {
      $config = $this->configFactory->get('cutoff_date.settings');
      $banner_text = $config->get('banner_text');
      $cache_tags = $config->getCacheTags();
    }

    if (empty($banner_text['value'])) {
      return [];
    }

    $build['content'] = [
      '#markup' => $banner_text['value'],
      '#cache' => [
        'tags' => $cache_tags ?? [],
      ],
    ];
    return $build;
  }

}
